{
****************************************************************
*  Name    : GLCD.BAS                                          *
*  Author  : David John Barker                                 *
*          : Steven Wright                                     *
*  Notice  : Copyright (c) 2006 Mecanique                      *
*          : All Rights Reserved                               *
*  Date    : 02/05/2006                                        *
*  Version : 1.2                                               *
*  Notes   : 1.2 Added CreateMask() for 1 bit Y fonts          *
*          : 1.3 Added support for S1D13700                    *
****************************************************************
}
Module GLCD

// validate model...
#option GLCD_MODEL = KS0108
#if Not (GLCD_MODEL in (KS0108, S1D10605, S1D15G00, SED1520, S1D13700, T6963c))
   #error GLCD_MODEL, "Invalid option. GLCD model not recognized."
#endif
   
// import KS0108 driver...
#if GLCD_MODEL = KS0108
Include "KS0108.bas"                
Public Dim
   Pos As KS0108.Pos, 
   Cls As KS0108.Cls,
   SetPixel As KS0108.SetPixel,
   WriteByte As KS0108.WriteByte,
   ReadByte As KS0108.ReadByte,
   GetPixel As KS0108.GetPixel
#endif

// import S1D10605 driver...
#if GLCD_MODEL = S1D10605
Include "S1D10605.bas"               
Public Dim
   Pos As S1D10605.Pos, 
   Cls As S1D10605.Cls,
   SetPixel As S1D10605.SetPixel,
   SetContrast As S1D10605.SetContrast,
   WriteByte As S1D10605.WriteByte,
   ReadByte As S1D10605.ReadByte,
   GetPixel As S1D10605.GetPixel,
   Sleep As S1D10605.Sleep,
   StandBy As S1D10605.StandBy   
#endif

// import S1D15G00 driver...
#if GLCD_MODEL = S1D15G00
Include "S1D15G00.bas"  
Public Dim
   Pos As S1D15G00.Pos, 
   Cls As S1D15G00.Cls,
   SetPixel As S1D15G00.SetPixel,
   SetContrast As S1D15G00.SetContrast,
   Fill As S1D15G00.Fill,
   Contrast As S1D15G00.Contrast,
   MakeColor As S1D15G00.MakeColor,
   SetImage As S1D15G00.SetImage,
   CopyImage As S1D15G00.CopyImage
#endif

// import SED1520 driver...
#if GLCD_MODEL = SED1520
Include "SED1520.bas"  
Public Dim
   Pos As SED1520.Pos, 
   Cls As SED1520.Cls,
   SetPixel As SED1520.SetPixel,
   WriteByte As SED1520.WriteByte,
   ReadByte As SED1520.ReadByte,
   GetPixel As SED1520.GetPixel
#endif

// import S1D13700 driver...
#if GLCD_MODEL = S1D13700
Include "S1D13700.bas"  
Public Dim
   Pos As S1D13700.Pos, 
   Cls As S1D13700.Cls,
   SetPixel As S1D13700.SetPixel,
   WriteByte As S1D13700.WriteByte,
   ReadByte As S1D13700.ReadByte,
   GetPixel As S1D13700.GetPixel,
   ShowLayer As S1D13700.ShowLayer,
   SwitchLayer As S1D13700.SwitchLayer,
   Fill As S1D13700.Fill,
   SetImage As S1D13700.SetImage
#endif

// import S1D13700 driver...
#if GLCD_MODEL = T6963c
Include "T6963c.bas"  
Public Dim
   Pos As T6963c.Pos, 
   Cls As T6963c.Cls,
   SetPixel As T6963c.SetPixel,
   WriteByte As T6963c.WriteByte,
   ReadByte As T6963c.ReadData
   //GetPixel As T6963c.GetPixel,
#endif
// import graphics and math...
Include "Graphics.bas"
Include "math.bas"

// import constants...
Public Const
   Width = GLCDWidth,
   Height = GLCDHeight

// text out alignment constants
Public Const
   taLeft = $01,
   taCenter = $02,
   taRight = $04
   
// make graphics vriables available for this
// module interface
Public Dim
   SetFont As Graphics.SetFont,
   TextWidth As Graphics.TextWidth,
   Pen As Graphics.Pen,
   Brush As Graphics.Brush,
   Font As Graphics.Font,
   TextAlign As Byte

// default is equal apect ratio, change in GLCD model core...
#option GLCD_ASPECT_RATIO = 100
Const AspectRatio = GLCD_ASPECT_RATIO 

{
****************************************************************************
* Name    : WriteImageByte (PRIVATE)                                       *
* Purpose : Write an image byte to GLCD at pixel position x,y              *
****************************************************************************
} 
#ifndef GLCD_SETIMAGE
#ifdef GLCD_PIXEL_01
Sub WriteImageByte(pX, pY As TXY, pValue As Byte)
   Dim Line, ShiftValue, YBit, YBitReverse As Byte
   Dim LastPosY As Byte
 
   LastPosY = Pos.y           // save y position
   YBit = pY Mod 8            // offset
   YBitReverse = 8 - YBit     // reverse offset
   Pos.y = pY / 8             // align to page
   Pos.x = pX                 // save x

   // get first byte...
   Line = ReadByte   

   // mask, if needed...
   If Pen.Mode = pmCopy Then
      Line = Line << YBitReverse
      Line = Line >> YBitReverse
   EndIf
   
   // merge then write data...   
   ShiftValue = pValue << YBit
   If Pen.Mode = pmXOR Then
      Line = Line Xor ShiftValue
   Else
      Line = Line Or ShiftValue
   EndIf
   WriteByte(Line)
    
   // get next byte...
   If YBit <> 0 Then
      Inc(Pos.y)
      Line = ReadByte
  
      // mask, if needed...
      If Pen.Mode = pmCopy Then
         Line = Line >> YBit
         Line = Line << YBit
      EndIf
   
      // merge then write data...
      ShiftValue = pValue >> YBitReverse
      If Pen.Mode = pmXOR Then
         Line = Line Xor ShiftValue
      Else
         Line = Line Or ShiftValue
      EndIf   
      WriteByte(Line) 
   EndIf
   
   // restore y position...
   Pos.y = LastPosY   
End Sub 
{
****************************************************************************
* Name    : SetImage                                                       *
* Purpose : BitBlit image to GLCD at pixel location x,y                    *
*         : NOTE - This subroutine only supports $01 (monochrome) bitmaps  *
*         :        for drivers with ReadByte and WriteByte implemented     *
****************************************************************************
} 
Public Sub SetImage(pX, pY As TXY, ByRefConst pImage() As Byte, pMode As Byte = cmCopy)
   Dim Image As TImage
   Dim x,y,Width,Height As Byte
   Dim Line As Byte
   Dim LastPen As TPen
   Dim Index As Word
   
   LastPen = Pen
   Pen.Mode = pMode
 
   // get bitmap width and height...
   Image.Width = pImage(1)
   Image.Height = pImage(2)
     
   // byte aligned bit blit... 
   Index = 4
   y = pY
   Height = Image.Height / 8
   While Height > 0
      x = pX
      Width = Image.Width
      While Width > 0
         WriteImageByte(x,y,pImage(Index))
         Inc(Index)
         Inc(x)
         Dec(Width)
      Wend
      Inc(y,8)
      Dec(Height)
   Wend 
         
   // non byte aligned bit blit...
   pY = y
   Image.Height = Image.Height Mod 8   
   If Image.Height > 0 Then
      x = pX
      Width = Image.Width
      While Width > 0     
         Line = pImage(Index)
         Height = Image.Height
         y = pY
         While Height > 0
            Pen.Color = Line.0
            SetPixel(x,y)
            Line = Line >> 1
            Dec(Height)
            Inc(y)
         Wend
         Inc(Index)
         Dec(Width)
         Inc(x)
      Wend
   EndIf   
   Pen = LastPen   
End Sub
#endif // GLCD_PIXEL_01
#endif // GLCD_SETIMAGE

{
****************************************************************************
* Name    : Line                                                           *
* Purpose : Draws a line using a Bresenham line drawing algorithm          *
****************************************************************************
}
#ifndef GLCD_LINE
Public Sub Line(x1, y1, x2, y2 As TXY)
   Dim P, diff As LongInt
   Dim incx, incy  As Integer
   Dim dx_ge_dy As Boolean
   Dim dy, dx, delta As Word
   Dim PenPos As Byte
   Dim Length As Byte
   
   // x direction...
   incx = 1
   If x1 > x2 Then
      incx = -1
   EndIf   
  
   // y direction
   incy = 1
   If y1 > y2 Then
      incy = -1
   EndIf   

   dx = abs(x2 - x1)
   dy = abs(y2 - y1)

   If dx >= dy Then
      dx_ge_dy = true
      dy = dy * 2
      p = dy - dx
      delta = dx
   Else
      dx_ge_dy = false
      dx = dx * 2
      p = dx - dy
      delta = dy
   EndIf   
   diff = p - delta

   Inc(delta)

   PenPos = 0
   Length = Pen.Style
   
   Repeat
      If Pen.Style > 0 Then
         If PenPos < Length Then
            SetPixel(x1, y1)
         EndIf
         Inc(PenPos)
         If PenPos > Length Then
            PenPos = 0
         EndIf
      Else
         SetPixel(x1, y1) 
      EndIf
      If P < 0 Then
         If dx_ge_dy Then
            Inc(p, dy)
            Inc(x1, incx)
         Else
            Inc(p, dx)
            Inc(y1, incy)
         EndIf   
      Else
         Inc(p, diff)
         Inc(x1, incx)
         Inc(y1, incy)
     EndIf
     Dec(delta)
   Until delta = 0 
End Sub
#endif // GLCD_LINE

{
****************************************************************************
* Name    : Plot (PRIVATE)                                                 *
* Purpose : Plots a set of ellipse quadrant points                         *
****************************************************************************
}
#ifndef GLCD_ELLIPSE
Sub Plot(pCX, pCY, pX, pY As TXY)
   SetPixel(pCX + pX, pCY + pY)
   SetPixel(pCX - pX, pCY + pY)
   SetPixel(pCX - pX, pCY - pY)
   SetPixel(pCX + pX, pCY - pY)
End Sub
{
****************************************************************************
* Name    : Ellipse                                                        *
* Purpose : Draws an ellipse                                               *
*         : A Fast Bresenham Type Algorithm For Drawing Ellipses           *
*         : John Kennedy                                                   *
*         : Mathematics Department, Santa Monica College                   *
*         : Santa Monica, CA 90405 - http://homepage.smc.edu/kennedy_john  *
****************************************************************************
}
Public Sub Ellipse(pCX, pCY, pXRadius, pYRadius As TXY)
  Dim x, y As TXY
  Dim XChange, YChange As LongInt
  Dim EllipseError As LongInt
  Dim TwoASquare, TwoBSquare As LongInt
  Dim StoppingX, StoppingY As LongInt
  Dim PenPos As Byte
  Dim Length As Byte

  Length = Pen.Style

  TwoASquare = 2 * pXRadius * pXRadius
  TwoBSquare = 2 * pYRadius * pYRadius
  x = pXRadius
  y = 0
  XChange = pYRadius * pYRadius * (1 - 2 * pXRadius)
  YChange = pXRadius * pXRadius
  EllipseError = 0
  StoppingX = TwoBSquare * pXRadius
  StoppingY = 0
  PenPos = 0
  While StoppingX >= StoppingY  
      If Pen.Style > 0 Then
         If PenPos < Length Then
            Plot(pCX, pCY, x, y)
         EndIf
         Inc(PenPos)
         If PenPos > Length Then
            PenPos = 0
         EndIf
      Else
         Plot(pCX, pCY, x, y)
      EndIf
      Inc(y)
      Inc(StoppingY, TwoASquare)
      Inc(EllipseError, YChange)
      Inc(YChange,TwoASquare)
      If (2 * EllipseError + XChange) > 0  Then
          Dec(x)
          Dec(StoppingX, TwoBSquare)
          Inc(EllipseError, XChange)
          Inc(XChange,TwoBSquare)
      EndIf
  Wend

  x = 0
  y = pYRadius
  XChange = pYRadius * pYRadius
  YChange = pXRadius * pXRadius * (1 - 2 * pYRadius)
  EllipseError = 0
  StoppingX = 0
  StoppingY = TwoASquare * pYRadius
  PenPos = 0
  While StoppingX <= StoppingY 
      If Pen.Style > 0 Then
         If PenPos < Length Then
            Plot(pCX, pCY, x, y)
         EndIf
         Inc(PenPos)
         If PenPos > Length Then
            PenPos = 0
         EndIf
      Else
         Plot(pCX, pCY, x, y)
      EndIf
      Inc(x)
      Inc(StoppingX, TwoBSquare)
      Inc(EllipseError, XChange)
      Inc(XChange,TwoBSquare)
      If (2 * EllipseError + YChange) > 0  Then
          Dec(y)
          Dec(StoppingY, TwoASquare)
          Inc(EllipseError, YChange)
          Inc(YChange,TwoASquare)
      EndIf
  Wend
End Sub
#endif // GLCD_ELLIPSE

{
****************************************************************************
* Name    : Circle                                                         *
* Purpose : Draws a circle, Y is scaled to compensate for aspect ratio     *
****************************************************************************
} 
#ifndef GLCD_CIRCLE
Public Sub Circle(pCX, pCY, pRadius As TXY)
   #if GLCD_ASPECT_RATIO <> 100
   Ellipse(pCX, pCY, pRadius, pRadius * AspectRatio / 100)
   #else
   Ellipse(pCX, pCY, pRadius, pRadius)
   #endif
End Sub
#endif // GLCD_CIRCLE

{
****************************************************************************
* Name    : HLine (PRIVATE)                                                *
* Purpose : Draws a horizontal line                                        *
****************************************************************************
}
#ifndef GLCD_RECTANGLE
Sub HLine(pX1, pX2, pY As TXY)
   Dim PenPos As Byte
   Dim Length As Byte
   PenPos = 0
   Length = Pen.Style
   While pX1 <= pX2
      If Pen.Style > 0 Then
         If PenPos < Length Then
            SetPixel(pX1,pY)
         EndIf
         Inc(PenPos)
         If PenPos > Length Then
            PenPos = 0
         EndIf
      Else
         SetPixel(pX1,pY)
      EndIf
      Inc(pX1)
   Wend   
End Sub
{
****************************************************************************
* Name    : VLine (PRIVATE)                                                *
* Purpose : Draw a vertical line                                           *
****************************************************************************
}
Sub VLine(pX, pY1, pY2 As TXY)
   Dim PenPos As Byte
   Dim Length As Byte
   PenPos = 0
   Length = Pen.Style
   While pY1 <= pY2
      If Pen.Style > 0 Then
         If PenPos < Length Then
            SetPixel(pX,pY1)
         EndIf
         Inc(PenPos)
         If PenPos > Length Then
            PenPos = 0
         EndIf
      Else
         SetPixel(pX,pY1)
      EndIf
      Inc(pY1)
   Wend   
End Sub
{
****************************************************************************
* Name    : Rectangle                                                      *
* Purpose : Draws a rectangle                                              *
****************************************************************************
} 
Public Sub Rectangle(px1, py1, px2, py2 As TXY)
   Dim Swap As TXY
   
   If px2 < px1 Then
      Swap = px1
      px1 = px2
      px2 = Swap
   EndIf

   If py2 < py1 Then
      Swap = py1
      py1 = py2
      py2 = Swap
   EndIf
   
   HLine(px1, px2, py1)
   VLine(px2, py1, py2)
   HLine(px1, px2, py2)
   VLine(px1, py1, py2) 
End Sub
#endif // GLCD_RECTANGLE

{
****************************************************************************
* Name    : Square                                                         *
* Purpose : Draws a square, Y is scaled to compensate for aspect ratio     *
****************************************************************************
} 
#ifndef GLCD_SQUARE
Public Sub Square(pX, pY, pSize As TXY)
   #if GLCD_ASPECT_RATIO <> 100
   Rectangle(pX, pY, pX + pSize, pY + pSize * AspectRatio / 100)
   #else
   Rectangle(pX, pY, pX + pSize, pY + pSize)
   #endif
End Sub
#endif // GLCD_SQUARE

{
****************************************************************************
* Name    : MoveTo                                                         *
* Purpose : Moves graphics cursor to given point before calling LineTo()   *
****************************************************************************
} 
Public Sub MoveTo(pX, pY As TXY)
   Pos.x = pX
   Pos.y = pY
End Sub
{
****************************************************************************
* Name    : LineTo                                                         *
* Purpose : Draws a line from the current graphics cursor position         *
****************************************************************************
} 
Public Sub LineTo(pX, pY As TXY)
   Line(Pos.x, Pos.y, pX, pY)
   Pos.x = pX
   Pos.y = pY
End Sub

// monchrome font rendering, WriteChar has not be implemented
// in main GLCD driver module...
#ifndef GLCD_WRITECHAR
#ifdef GLCD_PIXEL_01
Dim 
   FCharLocation,    // unmodified y (pixel) character location (set in WriteChar)
   FMaskYPosition,   // current mask y position
   FMaskYMax As TXY  // max y position
{
****************************************************************************
* Name    : ApplyFontStyle (PRIVATE)                                       *
* Purpose : Apply font rendering style                                     *
****************************************************************************
}   
Function ApplyFontStyle(pTarget, pSource As Byte) As Byte
   If Font.Style = fsXOR Then
      Result = pTarget Xor pSource
   Else
      Result = pTarget Or pSource
   EndIf
End Function
{
****************************************************************************
* Name    : CreateMask (PRIVATE)                                           *
* Purpose : Create a font mask - a 1 will preserve the background, 0 will  *
*         : clear                                                          *
****************************************************************************
} 
Function CreateMask() As Byte
   Dim FontHeight As Byte
   FontHeight = FCharLocation + Font.Height 
   result = $FF
   Repeat
      result = result >> 1
      result.7 = 1
      If (FMaskYPosition >= FCharLocation) And (FMaskYPosition < FontHeight) Then
         result.7 = 0
      EndIf
      Inc(FMaskYPosition)
   Until FMaskYPosition >= FMaskYMax
   Inc(FMaskYMax,8)
End Function
{
****************************************************************************
* Name    : WriteCharacterByte (PRIVATE)                                   *
* Purpose : Write a font character byte to GLCD                            *
****************************************************************************
} 
Sub WriteCharacterByte(pValue As Byte)
   Dim Line, YBit, YBitReverse As Byte
   Dim LastPosY As Byte
   
   LastPosY = Pos.y         // save y position
   YBit = Pos.y Mod 8       // offset
   YBitReverse = 8 - YBit   // reverse offset
   Pos.y = Pos.y / 8        // align to page

   // get data...
   Line = ReadByte
   
   // mask, if required...
   If Font.Style = fsNormal Then
      FMaskYPosition = LastPosY                 // current y position
      FMaskYMax = FMaskYPosition + YBitReverse  
      Line = Line And CreateMask
   EndIf
   
   // apply font style, then write to GLCD...
   Line = ApplyFontStyle(Line,pValue << YBit)
   WriteByte(Line)
   
   // get next byte...
   If YBit <> 0 Then
      Inc(Pos.y)
      Line = ReadByte
     
      // mask, if required...
      If Font.Style = fsNormal Then
         Line = Line And CreateMask
      EndIf
      
      // apply font style, then write to GLCD
      Line = ApplyFontStyle(Line, pValue >> YBitReverse)
      WriteByte(Line)
   EndIf
   
   // restore y position...
   Pos.y = LastPosY
End Sub 
{
****************************************************************************
* Name    : WriteChar (PRIVATE)                                            *
* Purpose : Write a single character to GLCD                               *
*         : This routine only supports fonts with a y scan Line ($8x xx)   *
****************************************************************************
} 
Sub WriteChar()
   Dim FontByte, FontWidth, FontHeight As Byte
   Dim Mask, LastStyle As Byte

   FCharLocation = Pos.y
   FontWidth = 0
   While FontWidth < Font.Width
      Pos.y = FCharLocation
      FontHeight = 0
      While FontHeight < Font.Height
         FontByte = ReadFontByte
         Inc(FontHeight, 8)
         LastStyle = Font.Style

         If Font.Style = fsInvert Then
            Font.Style = fsNormal
            If FontHeight > Font.Height Then
               Mask = $FF << 8 - (FontHeight - Font.Height)
               FontByte = FontByte Or Mask
            EndIf
            FontByte = Not FontByte               
         EndIf
         WriteCharacterByte(FontByte)         
         Font.Style = LastStyle
         Inc(Pos.y, 8)
      Wend
      Inc(Pos.x) 
      If Pos.x >= Width Then
         Pos.x = 0	
      EndIf
      Inc(FontWidth)
   Wend
   Pos.y = FCharLocation  
End Sub
#endif // GLCD_PIXEL_01

{
****************************************************************************
* Name    : WriteCharPixel (PRIVATE)                                       *
* Purpose : Write a character pixel to the GLCD                            *
****************************************************************************
} 
#ifdef GLCD_PIXEL_08
Sub WriteCharPixel(pChar As Byte, pPenColor As TColor)
   Pen.Color = pPenColor
   If Brush.Style = bsClear Then
      If pChar.0 = 1 Then
         SetPixel(Pos.x,Pos.y)
      EndIf
   Else
      If pChar.0 = 0 Then
         Pen.Color = Brush.Color
      EndIf
      SetPixel(Pos.x,Pos.y)
   EndIf   
End Sub
{
****************************************************************************
* Name    : WriteChar(PRIVATE)                                             *
* Purpose : Write a single character to GLCD                               *
*         : This routine only supports fonts with a y scan line ($8x xx)   *
*         : It's device independent, which makes it pretty slow - look at  *
*         : implementing a device specific routine in the GLCD driver      *
****************************************************************************
} 
Sub WriteChar()
   Dim x, y As TXY
   Dim FontByte, LastPosY As Byte
   Dim PenColor As TColor

   LastPosY = Pos.y
   PenColor = Pen.Color
   If Font.IsScanY Then
      x = 0
      Repeat
         Pos.y = LastPosY
         y = 0
         Repeat
            If y Mod 8 = 0 Then
               FontByte = ReadFontByte
            EndIf   
            WriteCharPixel(FontByte, PenColor)           
            FontByte = FontByte >> 1
            Inc(Pos.y)
            Inc(y)
         Until y = Font.Height
         Inc(Pos.x)
         Inc(x)
      Until x = Font.Width
   EndIf  
   Pen.Color = PenColor
   Pos.y = LastPosY
End Sub
#endif // GLCD_PIXEL_08
#endif // GLCD_WRITECHAR

{
****************************************************************************
* Name    : WriteFontChar                                                  *
* Purpose : Write a font character to the GLCD                             *
****************************************************************************
} 
Sub WriteFontChar(pChar As Char)
   LoadCharAddress(pChar)
   If Font.IsVariable Then
      Font.Width = ReadFontByte
   EndIf
   WriteChar  
End Sub
{
****************************************************************************
* Name    : WriteLetterSpacing                                             *
* Purpose :                                                                *
****************************************************************************
} 
Sub WriteFontSpace()
   Dim Index As Byte
   Index = Font.LetterSpacing
   While Index > 0
      LoadLetterSpacing
      WriteChar
      Dec(Index)
   Wend
End Sub
{
****************************************************************************
* Name    : WriteItem (OVERLOAD)                                           *
* Purpose : Writes a character to GLCD                                     *
****************************************************************************
} 
Sub WriteItem(pChar As Char)
   WriteFontChar(pChar)
End Sub
{
****************************************************************************
* Name    : WriteItem (OVERLOAD)                                           *
* Purpose : Writes a string to GLCD                                        *
****************************************************************************
} 
Sub WriteItem(pStr As String)
   FSR0 = @pStr
   While INDF0 <> 0
      WriteFontChar(POSTINC0)
      If Font.IsVariable And INDF0 <> 0 Then
         WriteFontSpace
      EndIf   
   Wend
End Sub
{
****************************************************************************
* Name    : SetLocationX (PRIVATE)                                         *
* Purpose : Set cursor x                                                   *
****************************************************************************
} 
Sub SetLocationX(pX As TXY)
   Pos.x = pX
End Sub
{
****************************************************************************
* Name    : SetLocationY (PRIVATE)                                         *
* Purpose : Set cursor y                                                   *
****************************************************************************
} 
Sub SetLocationY(pY As TXY)
   Pos.y = pY
End Sub
{
****************************************************************************
* Name    : Write (COMPOUND)                                               *
* Purpose : Write one or more items to GLCD                                *
****************************************************************************
} 
Public Compound Sub Write(WriteItem)
{
****************************************************************************
* Name    : WriteAt (COMPOUND)                                             *
* Purpose : Write one or more items at position x, y                       *
****************************************************************************
} 
Public Compound Sub WriteAt(SetLocationX, SetLocationY, WriteItem)
{
****************************************************************************
* Name    : WriteStr                                                       *
* Purpose : Write one item centred in x At position x, y                   *
****************************************************************************
} 
Public Sub WriteStr(pX, pY As TXY, pStr As String)
   Pos.x = pX
   Pos.y = pY
   If TextAlign = taCenter Then
      Dec(Pos.x,TextWidth(pStr) / 2)
   ElseIf TextAlign = taRight Then
      Dec(Pos.x,TextWidth(pStr))
   EndIf      
   WriteItem(pStr)
End Sub

// initialize the module
TextAlign = taLeft

